%Matlab script illustrating generation of simulated data for specified
%parameters, and fitting a hierarchy of state-space models
%expect this to take a couple of minutes
%LICENSE
% This file is part of rsstatespace.
%
%    rsstatespace is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    stochmap is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with rsstatespace.  If not, see <http://www.gnu.org/licenses/>.
%
%See gpl-3.0.txt for full information.

clear;

%specify true parameters
beta=[-0.1461;-3.7406;-0.0758];%parameters for model F fitted to region D2
epsilon=0.3603;%average measurement error variance over all selected models
eta=1.1731;%average process error variance over all selected models
nyears=19;%shortest series in the real data
tau=10;%middle of series
reps=9;%number of parametric bootstrap reps: small value for demonstration, use a much larger number, e.g. 999, for real analysis.
alpha=0.05;%significance level used to select models from real data
startval=3.7511;%first observation for region D2

%generate a simulated data set
vtrue='111';%indicates that all three beta coefficients are nonzero (model F)
X0sim=Xfromv(vtrue,[],tau,nyears);%generate explanatory variables X for simulated data
[simalpha,simy]=univariate_sim(nyears,startval,epsilon,eta,beta,X0sim);%simulate: simalpha is true values, simy is observations

%plot the simulated data
figure(1);
clf;
plot(1:nyears,simalpha,'b-',1:nyears,simy,'ro');
xlabel('Time');
ylabel('Value');
title('Simulated data');
legend({'simulated true values','simulated observations'});
hold on;
yl=get(gca,'ylim');
plot([tau tau],yl,'k--');%plot the change point

%fit all the models to the simulated data
%strings defining models
vA='000';
vB='010';
vC='100';
vD='110';
vE='101';
vF='111';

%start values
startA=0;
startB=[0 0]';
startC=[0 0]';
startD=[0 0 0]';
startE=[0 0 0]';
startF=[0 0 0 0]';

ntest=7;
years=1:nyears;
pvalues=zeros(ntest,1);
%cell array of test pairs. In each row, null model is first col,
%alternative model is second col
testpairs={...
    vD,vF;...%test i
    vE,vF;...%test ii
    vB,vD;...%test iii
    vC,vD;...%test iv
    vC,vE;...%test v
    vA,vB;...%test vi
    vA,vC;...%test vii
    };

%cell array of starting parameter values. In each row, null model is first col,
%alternative model is second col
startpairs={...
    startD,startF;...%test i
    startE,startF;...%test ii
    startB,startD;...%test iii
    startC,startD;...%test iv
    startC,startE;...%test v
    startA,startB;...%test vi
    startA,startC;...%test vii
    };

for j=1:ntest%fit and compare all the models
    disp(['test ',num2str(j)]);
    [LRmax,p,repsdone,negltau0,negltau1,bestshift0,bestshift1,fitset0,fitset1,timetaken]...
        =kalman_diffuse_shiftgeneral_bootone(simy,reps,testpairs{j,1},testpairs{j,2},startpairs{j,1},startpairs{j,2});
    sresult(j)=struct(...
        'seriesnames','simulated test data',...
        'years',years,...
        'data',simy,...
        'test',j,...
        'nullmodel',testpairs{j,1},...
        'alternativemodel',testpairs{j,2},...
        'nullstart',startpairs{j,1},...
        'alternativestart',startpairs{j,2},...
        'LRmax',LRmax,...
        'p',p,...
        'repsdone',repsdone,...
        'negltau0',negltau0,...
        'negltau1',negltau1,...
        'bestshift0',bestshift0,...
        'bestshift1',bestshift1,...
        'fitset0',fitset0,...
        'fitset1',fitset1,...
        'timetaken',timetaken...
     );%store results
    pvalues(j)=sresult(j).p;%construct vector of p-values
end
    
%which model did we select?
figure(2);
clf;
plotselectstatespace(pvalues,0.05,0,'Model selection');

%produce a diagnostic plot for the selected model
plausible=selectstatespace(pvalues,0.05);%tests at level 0.05
pm=find(plausible);
modelnames={'A','B','C','D','E','F'};
for k=1:length(pm) %there may be more than one model
    %extract the parameters we need
    switch pm(k) %map to relevant model in resultstruct
        case 1 %model A: no growth, no step, no change in growth
            tau=1;
            v='000';
            kalmanPCdiagnostics(sresult(7).fitset0(tau).et,sresult(7).fitset0(tau).at,sresult(7).data,sresult(7).years,10,v,[],sresult(7).fitset0(tau).beta,['simulated data',', model ',modelnames{pm(k)}]);

            case 2 %model B: no growth, step, no change in growth
                tau=sresult(6).bestshift1;
                v='010';
                kalmanPCdiagnostics(sresult(6).fitset1(tau).et,sresult(6).fitset1(tau).at,sresult(6).data,sresult(6).years,10,v,tau,sresult(6).fitset1(tau).beta,['simulated data',', model ',modelnames{pm(k)}]);
                
            case 3 %model C: growth, no step, no change in growth
                tau=1;
                v='100';
                kalmanPCdiagnostics(sresult(4).fitset0(tau).et,sresult(4).fitset0(tau).at,sresult(4).data,sresult(4).years,10,v,[],sresult(4).fitset0(tau).beta,['simulated data',', model ',modelnames{pm(k)}]);
                
            case 4 %model D: growth, step, no change in growth
                tau=sresult(3).bestshift1;
                v='110';
                kalmanPCdiagnostics(sresult(3).fitset1(tau).et,sresult(3).fitset1(tau).at,sresult(3).data,sresult(3).years,10,v,tau,sresult(3).fitset1(tau).beta,['simulated data',', model ',modelnames{pm(k)}]);
                
            case 5 %model E: growth, no step, change in growth
                tau=sresult(2).bestshift0;
                v='101';
                kalmanPCdiagnostics(sresult(2).fitset0(tau).et,sresult(2).fitset0(tau).at,sresult(2).data,sresult(2).years,10,v,tau,sresult(2).fitset0(tau).beta,['simulated data',', model ',modelnames{pm(k)}]);
                
            case 6 %model F: growth, step, change in growth
                tau=sresult(1).bestshift1;
                v='111';
                kalmanPCdiagnostics(sresult(1).fitset1(tau).et,sresult(1).fitset1(tau).at,sresult(1).data,sresult(1).years,10,v,tau,sresult(1).fitset1(tau).beta,['simulated data',', model ',modelnames{pm(k)}]);
    end
end