function kalmanPCdiagnostics(et,at,Y,years,maxlag,v,tau,beta,infostr)
%diagnostics for kalman filter models for first PC
%SYNTAX
%kalmanPCdiagnostics(et,at,Y,maxlag,v,tau,beta,infostr)
%INPUTS
%et is standardized residuals
%at is filtered states
%Y is observed data
%years is the years in which observations were made
%maxlag is largest autocorrelation to look out
%v is a string indicating the model, with three characters (each either 1
%or 0) representing growth, step, change in growth
%tau is location of best shift (can be empty)
%beta is a vector of parameters for the deterministic part of the model,
%corresponding to the nonzero elements of v, in the order they appear
%infostr (optional) is a string to addd to the label for the first panel
%OUTPUTS
%time plot of observed data (circles), filtered state (dashed line), best shift location as
%dash-dot line if specified, deterministic skeleton (solid line)
%three plots of standardized residuals: 
%time plot (should show no systematic
%change in level or variability). Solid line is at zero. Best shift
%location as dash-dot line if specified.
%quantile plot (should look like a straight line)
%correlogram (should not be any large autocorrelations). Solid line is at
%zero, dashed lines are approximate 95% significance bounds.
%USES
%ACF.m
%Xfromv.m
%LICENSE
% This file is part of rsstatespace.
%
%    rsstatespace is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    stochmap is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with rsstatespace.  If not, see <http://www.gnu.org/licenses/>.
%
%See gpl-3.0.txt for full information.

if nargin<9
    infostr=[];
end

%maximize the figure: better use of white space in eps files
fullscreen = get(0,'ScreenSize');
figure('Position',[0 -50 fullscreen(3) fullscreen(4)]);
clf;

labelsize=12;
ticksize=14;

%deterministic skeleton
n=length(years);
if strcmp(v,'000') || strcmp(v,'100')%no shift in the model
    %construct explanatory variable matrix
    if strcmp(v,'100')
       X=ones(n,1);%add term for constant growth rate 
    else
       X=zeros(n,1);%no growth rate
    end
else
    X=Xfromv(v,[],tau,n);%construct explanatory variable matrix
end
if isempty(beta)
    D=[NaN;Y(1)*ones(n-1,1)];
else
    psi=X*beta;
    D=Y(1)+cumsum(psi);
    D=[NaN;D(1:end-1)];
end

subplot(2,2,1);
plot(years,Y,'ko',years,at,'k--',years,D,'k-');
xlabel('Year','fontsize',labelsize);
ylabel('PC 1','fontsize',labelsize);
set(gca,'fontsize',ticksize)
xl=get(gca,'xlim');
yl=get(gca,'ylim');
if isempty(infostr)
    lt='(a)';
else
    lt=['(a) ',infostr];
end
text(xl(1)+0.1*(xl(2)-xl(1)),yl(2)-0.1*(yl(2)-yl(1)),lt,'fontsize',labelsize,'fontweight','bold');
set(gca,'xlim',xl,'ylim',yl);
if ~isempty(tau)%display shift location
    hold on;
    plot([tau+years(1)-1,tau+years(1)-1],yl,'k-.');
end;

subplot(2,2,2);
plot(years(2:end),et(2:end),'k-');%time plot: note et(1) is NaN because we have diffuse initialization
hold on;
plot([years(2),years(length(years))],[0 0],'k-');
xlabel('Year','fontsize',labelsize);
ylabel('Standardized residual','fontsize',labelsize);
set(gca,'fontsize',ticksize)
xl=get(gca,'xlim');
yl=get(gca,'ylim');
text(xl(1)+0.1*(xl(2)-xl(1)),yl(2)-0.1*(yl(2)-yl(1)),'(b)','fontsize',labelsize,'fontweight','bold');
set(gca,'xlim',xl,'ylim',yl);
if ~isempty(tau)%display shift location
    hold on;
    plot([tau+years(1)-1,tau+years(1)-1],yl,'k-.');
end;

subplot(2,2,3);
h=qqplot(et(2:end));%QQ plot
for i=1:length(h)
   set(h(i),'color','k','markeredgecolor','k'); 
end
set(h(3),'linestyle','--');
box on;
title('');
xlabel('Quantiles of standard normal','fontsize',labelsize);
ylabel('Quantiles of standardized residuals','fontsize',labelsize);
set(gca,'fontsize',ticksize)
xl=get(gca,'xlim');
yl=get(gca,'ylim');
text(xl(1)+0.1*(xl(2)-xl(1)),yl(2)-0.1*(yl(2)-yl(1)),'(c)','fontsize',labelsize,'fontweight','bold');
set(gca,'xlim',xl,'ylim',yl);

subplot(2,2,4);
[r,sigbounds]=ACF(et(2:end),maxlag);
plot(r,'k.');%correlogram
hold on;
for lag=1:maxlag
    plot([lag lag],[0 r(lag)],'k-');
end;
plot([1,maxlag],[0 0],'k-');
plot([1 maxlag],[sigbounds(1) sigbounds(1)],'k--');
plot([1 maxlag],[sigbounds(2) sigbounds(2)],'k--');
xlabel('Lag (years)','fontsize',labelsize);
ylabel('Sample autocorrelation','fontsize',labelsize);
set(gca,'fontsize',ticksize)
xl=get(gca,'xlim');
yl=get(gca,'ylim');
text(xl(1)+0.1*(xl(2)-xl(1)),yl(2)-0.1*(yl(2)-yl(1)),'(d)','fontsize',labelsize,'fontweight','bold');
set(gca,'xlim',xl,'ylim',yl);
