function [negldc,sigma2e,et,at]=kalman_diffuse2(theta,Y,X)
%univariate kalman filter for population growth with drivers, using
%concentrated diffuse log likelihood. This version: no separate population
%growth rate parameter (see notes).
%SYNTAX
%[negldc,sigma2e,et,at]=kalman_diffuse2(theta,Y,X)
%INPUTS
%theta is a k+1 x 1 vector of parameters:
%k explanatory variable coefficients and
%process error/measurement error q (log transformed, because it can't be negative)
%n x 1 matrix of data Y
%n x k matrix of drivers X
%OUTPUTS
%concentrated diffuse negative log likelihood negldc
%sigma2e is estimated measurement error, given q
%et is standardized residuals (a vector of values for times 2 to n, where n
%is the number of observations. NaN at time 1, where it isn't meaningfully defined)
%at is vector of filtered state at time t (NaN at time 1, where it isn't
%meaningfully defined)
%USES
%kalman_diffuse_backparms2.m
%NOTES
%Doesn't handle missing data (although it could do).
%Univariate Y (although it could easily be extended to multivariate).
%To include population growth, make one column of X be ones, then the
%corresponding beta is population growth rate (doing it this way makes it
%easier to set population growth rate to zero).
%Based closely on chapter 2 of Durbin, J. and Koopman, S.J. (2001). Time
%series analysis by state space methods. Oxford University Press.
%LICENSE
% This file is part of rsstatespace.
%
%    rsstatespace is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    stochmap is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with rsstatespace.  If not, see <http://www.gnu.org/licenses/>.
%
%See gpl-3.0.txt for full information.


n=length(Y);
[beta,q]=kalman_diffuse_backparms2(theta);%back transform parameters
if ~isempty(X)
    psi=X*beta;%deterministic growth term X*beta
else
    psi=zeros(n,1);%no deterministic growth term
end

sigma2e=0;%this will be estimate of measurement error conditional on q
negldc=0;%this will be concentrated diffuse log likelihood
et=zeros(n,1);%this will be standardized residuals
at=zeros(n+1,1);%this will be filtered states
et(1)=NaN;
at(1)=NaN;

at(2)=Y(1)+psi(1);%initialize at t=2, using diffuse initialization
P=1+q;
for t=2:n%for times 2 to n
   v=Y(t)-at(t);%residual
   F=P+1;%Ft* from page 32 in Durbin and Koopman
   K=P/F;
   at(t+1)=at(t)+psi(t)+K*v;
   P=P*(1-K)+q;%Pt* from page 32 in Durbin and Koopman
   sigma2e=sigma2e+v^2/F;%summation as in eq 2.45, Durbin and Koopman p. 32
   negldc=negldc+log(F);
   et(t)=v/sqrt(F);%standardized residual: Durbin and Koopman p. 33
end
at=at(1:n);%discard the filtered state at time n+1
sigma2e=sigma2e/(n-1);
et=et/sqrt(sigma2e);%we have Ft*=Ft/sigma2e above
negldc=(negldc+(n-1)*(1+log(sigma2e))+n*log(2*pi))/2;%equation 2.46, Durbin and Koopman p. 32

% figure(1);
% clf;
% plot(1:n,Y,'ko',1:n,at,'r-');
