function [negldc,beta,s2epsilon,s2eta,et,at,exitflag,optoutput,grad,hessian,theta]=kalman_diffuse_est2(data,X,logplus,startvals)
%Kalman filter for univariate population growth model:
%alpha(t+1)=alpha(t)+\sum_i beta_i*x_i(t)+eta(t)
%y(t)=alpha(t)+epsilon(t)
%where alpha(t) is the true value of the system (log abundance) at time t
%beta_i is coefficient of ith explanatory variable x_i(t)
%eta is Gaussian noise: mean zero, covariance Sigma
%y(t) is observation at time t
%epsilon(t) is Gaussian: mean zero, covariance H assumed to be diagonal
%y is observations (n rows, 1 column)
%This version: no separate growth rate parameter c. Specify it as a
%coefficient for an explanatory variable (see notes)
%SYNTAX
%[negldc,beta,s2epsilon,s2eta,et,at,exitflag,optoutput,grad,hessian,theta]=kalman_diffuse_est2(data,X,logplus,startvals)
%INPUTS
%data is a  vector of observations at each time step (row) 1...n
%if logplus (optional) is 1, use log(observations+1) instead of log
%(observations, default) as alpha.
%X is a matrix of k explanatory variables (n rows, k columns)
%startvals (optional) is transformed set of parameters [beta, log q] to use as a starting
%point (format as theta output)
%OUTPUTS
%negldc is negative concentrated diffuse log likelihood
%beta is estimated coefficients of explanatory variables (one row per variable)
%s2epsilon is estimated measurement error variance
%s2eta is estimated process error variance
%et is standardized residuals for times 1...n (NaN for time 1)
%at is filtered state at times 1...n (NaN for time 1)
%exitflag is optimizer termination status
%optoutput is a structure of optimizer output
%grad is the gradient at the ML estimate (for the transformed parameters theta, explained below)
%theta is the transformed parameter estimates in the order beta, log q,
%where q is s2eta/s2epsilon
%USES
%kalman_diffuse2.m
%kalman_diffuse_backparms2.m
%NOTES
%To include a mean growth rate c, specify a column of X that is entirely
%ones.
%LICENSE
% This file is part of rsstatespace.
%
%    rsstatespace is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    stochmap is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with rsstatespace.  If not, see <http://www.gnu.org/licenses/>.
%
%See gpl-3.0.txt for full information.

if nargin<3
    logplus=0;
end;
if nargin<4
    startvals=[];
end;

Y=log(data+logplus);
n=length(Y);
k=size(X,2);
nvar=k+2;
options=optimset('fminunc');
options=optimset(options,'display','off','largescale','off','MaxFunEvals',10000*nvar,'MaxIter',10000);

if isempty(startvals)
    %Start with some reasonable estimates for variances
    %based on Holmes, E. E. (2004). Ecological Applications 14:1272-1292, Appendix A. http://www.esapubs.org/archive/appl/A014/023/appendix-A.htm
    %(these do seem to work well)
    tmp1=var(Y(2:n)-Y(1:(n-1)));
    tmp4=var(Y(5:n)-Y(1:(n-4)));
    sigma2est=max(1e-6,(tmp4-tmp1)/3);%the max ensures we don't end up with negative starting values for variances
    sigma2npest=max(1e-6,var(Y(2:n)-Y(1:(n-1)))-sigma2est)/2;
    startvals=[zeros(k,1);log(sigma2npest/sigma2est)];
end

[theta,negldc,exitflag,optoutput,grad,hessian]=fminunc(@kalman_diffuse2,startvals,options,Y,X);%quasi-Newton
[beta,q]=kalman_diffuse_backparms2(theta);%back-transform parameters
[negldc,s2epsilon,et,at]=kalman_diffuse2(theta,Y,X);%need to call this once more to get s2epsilon, et, at
s2eta=q*s2epsilon;
end
