function [negltau,bestshift,fitset,timetaken]=kalman_diffuse_shiftgeneral(data,v,X,logplus,startvals)
%Estimate location of change point in population dynamics, for a
%univariate stochastic exponential model, possibly with explanatory
%variables
%Change point could be step, change in growth rate, or both
%SYNTAX
%[negltau,bestshift,fitset,timetaken]=kalman_diffuse_shiftgeneral(data,v,X,logplus,startvals)
%INPUTS
%data is a column vector of observations at each time
%v is a string specifying how the shift works: 3 elements. First
%element is 1 if there is a nonzero population growth rate, 0 otherwise. Second
%element is 1 if there is a step in abundance at unknown time tau, 0
%otherwise. Third element is 1 if there is a change in population growth
%rate at time tau, 0 otherwise. e.g. v='010' for no growth, step change, no
%change in growth.
%X is explanatory variables (in columns), excluding terms for base growth
%rate and shift.
%Use log(observations+logplus) as transformed observations
%startvals (optional) is starting values for parameters
%(otherwise use defaults from kalman_diffuse_est2.m). Should be supplied in the order
%[beta log q], where beta is coefficients for explanatory variables (the
%ones specified by the nonzero elements of v, then any extras in X)
%log q is log of ratio of process to measurement error
%OUTPUTS
%negltau is a vector of negative log likelihoods for models with
%shifts at times tau=1...n (will all be the same if there is no shift in
%the model)
%bestshift is the shift location corresponding to the smallest log likelihood. 
%If the model contains a shift, possible values are 2 to n, where n is the
%length of the time series. If the model does not contain a shift,
%bestshift is set arbitrarily to 1.
%fitset is a structure array, where the ith element is a structure containing the
%output of kalman_diffuse_est2 for a model with a shift at location i (n
%elements, but the first is empty because a shift at location 1 isn't
%meaningful).
%timetaken is clock time used
%USES
%kalman_diffuse_est2.m
%Xfromv.m
%NOTES
%Possible speedup: can we skip calculating the hessian for all shift points?
%LICENSE
% This file is part of rsstatespace.
%
%    rsstatespace is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    stochmap is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with rsstatespace.  If not, see <http://www.gnu.org/licenses/>.
%
%See gpl-3.0.txt for full information.

tzero=clock;
if nargin<5
    startvals=[];
end;

%initialize fitset as an array of structures of the right size
n=length(data);
fitset(n).negldc=[];
fitset(n).beta=[];
fitset(n).s2epsilon=[];
fitset(n).s2eta=[];
fitset(n).et=[];
fitset(n).at=[];
fitset(n).exitflag=[];
fitset(n).optoutput=[];
fitset(n).grad=[];
fitset(n).hessian=[];
fitset(n).theta=[];
negltau=zeros(1,n);

if strcmp(v,'000') || strcmp(v,'100')%no shift in the model
    %construct explanatory variable matrix
    if strcmp(v,'100')
       Xfull=[ones(n,1),X];%add term for constant growth rate 
    else
       Xfull=X;%no growth rate
    end
    [negldc,beta,s2epsilon,s2eta,et,at,exitflag,optoutput,grad,hessian,theta]=kalman_diffuse_est2(data,Xfull,logplus,startvals);%fit model once
    bestshift=1;%arbitrary choice of bestshift
    for s=1:n%all elements of fitset the same
        fitset(s)=struct('negldc',negldc,'beta',beta,'s2epsilon',s2epsilon,'s2eta',s2eta,'et',et,'at',at,'exitflag',exitflag,'optoutput',optoutput,'grad',grad,'hessian',hessian,'theta',theta);
        negltau(s)=negldc;
    end
else%there is some kind of shift in the model
    bestshift=0;
    bestnegl=Inf;
    negltau(1)=NaN;%we can't have a shift at time 1
    for s=2:n%for each time from 2 to n
        Xfull=Xfromv(v,X,s,n);%construct explanatory variable matrix
        [negldc,beta,s2epsilon,s2eta,et,at,exitflag,optoutput,grad,hessian,theta]=kalman_diffuse_est2(data,Xfull,logplus,startvals);%fit model with this shift
        fitset(s)=struct('negldc',negldc,'beta',beta,'s2epsilon',s2epsilon,'s2eta',s2eta,'et',et,'at',at,'exitflag',exitflag,'optoutput',optoutput,'grad',grad,'hessian',hessian,'theta',theta);
        negltau(s)=negldc;
        if negldc<bestnegl%better than any shift location found so far
            bestshift=s;
            bestnegl=negldc;
        end;
    end
end
timetaken=etime(clock,tzero);
