function [LRmax,p,repsdone,negltau0,negltau1,bestshift0,bestshift1,fitset0,fitset1,timetaken]=kalman_diffuse_shiftgeneral_bootone(data,reps,v0,v1,startvals0,startvals1)
%fit two univariate Kalman filter models to a single time series (e.g. first PC), possibly with change in dynamics at an unknown time, and get p-value for null model 0 against alternative model 1
%by parametric bootstrap
%SYNTAX
%timetaken=kalman_diffuse_shiftgeneral_bootone(data,reps,v0,v1,startvals0,startvals1)
%INPUTS
%data: the univariate time series (assume the specified models apply
%directly to this, not to the log of it)
%reps: number of parametric bootstrap reps to do (e.g. 99, 999) on each region
%v0 and v1 are strings containing three elements, each 1 or 0, specifying
%how the deterministic part of population growth works in the null and
%alternative models respectively. The elements are growth rate, step
%change, change in growth rate, each with 1 if that component is present
%and 0 otherwise.
%startvals0 and startvals1:
%starting values for estimation under null and alternative models. Column vector of [beta log q], 
%where beta is a vector of parameters for deterministic component of
%population growth (length depends on what the model is)
%log q is ratio of process to measurement error.
%OUTPUTS
%timetaken is clock time used
%USES
%kalman_diffuse_shiftgeneral.m
%Xfromv.m
%univariate_sim.m
%LICENSE
% This file is part of rsstatespace.
%
%    rsstatespace is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    stochmap is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with rsstatespace.  If not, see <http://www.gnu.org/licenses/>.
%
%See gpl-3.0.txt for full information.

tzero=clock;
n=length(data);
alpha1=data(1);%we don't actually estimate alpha1 when we use diffuse initialization, but if we do this, then a(2) has the right expected value
[negltau0,bestshift0,fitset0]=kalman_diffuse_shiftgeneral(exp(data),v0,[],0,startvals0);%fit the null model: use exp(data) because kalman_diffuse_shiftgeneral logs it again
[negltau1,bestshift1,fitset1]=kalman_diffuse_shiftgeneral(exp(data),v1,[],0,startvals1);%fit the alternative model
bsf0=fitset0(bestshift0);
LRmax=2*(negltau0(bestshift0)-negltau1(bestshift1));%likelihood ratio statistic
LRboot=zeros(reps,1);

for r=1:reps%for each parametric bootstrap rep:
    X0sim=Xfromv(v0,[],bestshift0,n);%generate X for the null model
    [alpha,y]=univariate_sim(n,alpha1,bsf0.s2epsilon,bsf0.s2eta,bsf0.beta,X0sim);%simulate under null model
    bootdata=exp(y);
    try
        [negltau0r,bestshifti0r]=kalman_diffuse_shiftgeneral(bootdata,v0,[],0,startvals0);%fit the null model to bootstrap data
        [negltau1r,bestshifti1r]=kalman_diffuse_shiftgeneral(bootdata,v1,[],0,startvals1);%fit the alternative model to bootstrap data
        LRboot(r)=2*(negltau0r(bestshifti0r)-negltau1r(bestshifti1r));%bootstrap likelihood ratio statistic
    catch%it's possible that the optimization will completely fail in some cases
        LRboot(r)=NaN;
    end
end%end bootstrap rep
k=sum(LRboot>=LRmax);%get p-value from distribution of bootstrapped likelihood ratios
repsdone=sum(~isnan(LRboot));%there may be some reps that completely failed
p=(k+1)/(repsdone+1);%bootstrap p-value: Davison and Hinkley p148
timetaken=etime(clock,tzero);