function plotselectstatespace(pvalues,alpha,newfig,titletext)
%plot the nesting structure and selected model for the six state-space
%models we considered. The nesting structure and coordinates are hard
%coded.
%SYNTAX
%plotselectstatespace(pvalues,alpha,newfig,title)
%INPUTS
%pvalues is a vector of 7 p-values for the comparisons of nested models
%alpha is the size for each test
%newfig (optional) is 1 if we want this in a new figure (default), or 0 if
%not (e.g. if it's part of a subplot)
%titletext is an optional title
%OUTPUTS
%A figure showing the nesting structure and p-values.
%grey lines for paths with p-values <=alpha
%USES
%selectstatespace.m
%arrow.m (from Matlab Central File Exchange) if it exists in search path

if nargin<3
    newfig=1;
end
if nargin<4
    titletext=[];
end
if newfig
    figure(1);
    clf;
end

hold on;
axis equal;
axis off;
plausible=selectstatespace(pvalues,alpha);%find out which models are plausible

Y=[0,1,1,2,2,3];%y coordinate of circle centres
X=[1,0,2,0,2,1];%x coordinate of circle centres

%draw circles: bold if plausible model
modelnames={'A','B','C','D','E','F'};
radius=0.2;
modelcircle(X,Y,modelnames,radius,plausible);

%draw model cartoons
plot([X(1)+radius+0.05;X(1)+radius+0.35],[Y(1);Y(1)],'k-');%flat
plot([X(2)+radius+0.05;X(2)+radius+0.2;X(2)+radius+0.2;X(2)+radius+0.35],[Y(2);Y(2);Y(2)+0.05;Y(2)+0.05],'k-');%step
plot([X(3)+radius+0.05;X(3)+radius+0.35],[Y(3);Y(3)+0.1],'k-');%trend
plot([X(4)+radius+0.05;X(4)+radius+0.2;X(2)+radius+0.2;X(2)+radius+0.35],[Y(4);Y(4)+0.05;Y(4)+0.1;Y(4)+0.15],'k-');%step+trend
plot([X(5)+radius+0.05;X(5)+radius+0.2;X(5)+radius+0.35],[Y(5);Y(5)+0.05;Y(5)+0.2],'k-');%trend with change
plot([X(6)+radius+0.05;X(6)+radius+0.2;X(6)+radius+0.2;X(6)+radius+0.35],[Y(6);Y(6)+0.05;Y(6)+0.1;Y(6)+0.25],'k-');%step+trend

%draw arrows
axis(axis);
modelarrow(X,Y,3,1,radius,pvalues(7),alpha);
modelarrow(X,Y,2,1,radius,pvalues(6),alpha);
modelarrow(X,Y,4,2,radius,pvalues(3),alpha);
modelarrow(X,Y,4,3,radius,pvalues(4),alpha);
modelarrow(X,Y,5,3,radius,pvalues(5),alpha);
modelarrow(X,Y,6,4,radius,pvalues(1),alpha);
modelarrow(X,Y,6,5,radius,pvalues(2),alpha);

%label arrows with p-values
text(1.6,0.5,num2str(pvalues(7),'%.3f'));
text(0.1,0.5,num2str(pvalues(6),'%.3f'));
text(2.05,1.5,num2str(pvalues(5),'%.3f'));
text(1.2,1.5,num2str(pvalues(4),'%.3f'));
text(-0.4,1.5,num2str(pvalues(3),'%.3f'));
text(1.6,2.5,num2str(pvalues(2),'%.3f'));
text(0.15,2.5,num2str(pvalues(1),'%.3f'));

if ~isempty(titletext)
    title(titletext,'fontsize',18);
end
end

%draw an arrow from model1 to model2 that doesn't cut the circle
function modelarrow(X,Y,model1,model2,radius,pval,alpha)
    h=0.01;
    deltaX=X(model1)-X(model2);
    deltaY=Y(model1)-Y(model2);
    theta=atan(deltaY/deltaX);
    vy=(radius+h)*sin(theta);
    vx=(radius+h)*cos(theta);
    if (deltaX<0 || deltaY<0)
        vy=-vy;
        vx=-vx;
    end
    if exist('arrow')%we have arrow function
        h=arrow([X(model1)-vx,Y(model1)-vy],[X(model2)+vx,Y(model2)+vy]);
        if pval<=alpha%grey lines for paths we can't follow
            set(h,'facecolor',[0.65 0.65 0.65],'edgecolor',[0.65 0.65 0.65]);
        end
    else%just draw line
        h=plot([X(model1)-vx,X(model2)+vx],[Y(model1)-vy,Y(model2)+vy],'k-');
        if pval<=alpha%grey lines for paths we can't follow
            set(h,'color',[0.65 0.65 0.65]);
        end
        
    end
end

%draw a circle for each model
function modelcircle(X,Y,modelnames,radius,plausible)
    for ii=1:length(X)
        Center=[X(ii),Y(ii)];
        h=plot(radius/2*exp(1i*linspace(0,2*pi,1000))+Center*[1 1i].','k-');
        if plausible(ii)
            set(h,'linewidth',2);
        end
        h=text(X(ii)-0.14*radius,Y(ii)-0.06*radius,modelnames{ii});
        if plausible(ii)
            set(h,'fontweight','bold');
        end

    end
end