function [alpha,y]=univariate_sim(n,alpha1,sigma2epsilon,sigma2eta,beta,X)
%simulate a univariate linear Gaussian state space model, possibly with
%explanatory variables
%the model is
%alpha(t+1)=alpha(t)+beta*x(t)+eta(t), where eta(t) is normal(0, sigma2eta)
%y(t)=x(t)+epsilon(t), where epsilon(t) is normal(0,sigma2epsilon)
%e.g. for log of an exponentially growing population with noise in both
%process and observation, and with a driver effect
%we could estimate parameters from exp(y), the original counts, using
%kalman_diffuse_est2.m
%SYNTAX
%[alpha,y]=univariate_sim(n,alpha1,sigma2epsilon,sigma2eta,beta,X)
%INPUTS
%n is number of time steps to simulate
%alpha1 is initial true state
%sigma2epsilon is measurement error variance
%sigma2eta is process error variance
%beta (optional) is a vector of k explanatory variable coefficients. If not
%given, no explanatory variables are used
%x (optional: specify if beta is given) is matrix of explanatory variables (n x k)
%OUTPUTS
%alpha is a vector of true states (initial value alpha0): n x 1
%y is a vector of observations: n x 1
%LICENSE
% This file is part of rsstatespace.
%
%    rsstatespace is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    stochmap is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with rsstatespace.  If not, see <http://www.gnu.org/licenses/>.
%
%See gpl-3.0.txt for full information.

if nargin<5
    beta=[];
    X=[];
end;

if size(beta,1)==1%turn into column vector if given as row vector
    beta=beta';
end;

alpha=zeros(n,1);
y=zeros(n,1);
alpha(1)=alpha1;
seps=sqrt(sigma2epsilon);
seta=sqrt(sigma2eta);
y(1)=alpha(1)+seps*randn;
for i=1:n-1
    alpha(i+1)=alpha(i)+seta*randn;%stochastic part of state equation
    if ~isempty(beta)
        alpha(i+1)=alpha(i+1)+X(i,:)*beta;%explanatory variables
    end;
    y(i+1)=alpha(i+1)+seps*randn;%observation
end;
